@@ -4,7 +4,7 @@
  * This file is part of msmtp, an SMTP client.
  *
  * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008, 2010, 2011, 2012,
- * 2014, 2015, 2016, 2018, 2019
+ * 2014, 2015, 2016, 2018, 2019, 2020
  * Martin Lambers <marlam@marlam.de>
  * Martin Stenberg <martin@gnutiken.se> (passwordeval support)
  * Scott Shumate <sshumate@austin.rr.com> (aliases support)
@@ -93,16 +93,19 @@ account_t *account_new(const char *conffile, const char *id)
     a->tls_nocertcheck = 0;
     a->tls_min_dh_prime_bits = -1;
     a->tls_priorities = NULL;
+    a->tls_host_override = NULL;
     a->logfile = NULL;
     a->logfile_time_format = NULL;
     a->syslog = NULL;
     a->aliases = NULL;
     a->proxy_host = NULL;
     a->proxy_port = 0;
-    a->add_missing_from_header = 1;
-    a->add_missing_date_header = 1;
+    a->set_from_header = 2;
+    a->set_date_header = 2;
     a->remove_bcc_headers = 1;
+    a->undisclosed_recipients = 0;
     a->source_ip = NULL;
+    a->socketname = NULL;
     return a;
 }
 
@@ -178,6 +181,8 @@ account_t *account_copy(account_t *acc)
         a->tls_min_dh_prime_bits = acc->tls_min_dh_prime_bits;
         a->tls_priorities =
             acc->tls_priorities ? xstrdup(acc->tls_priorities) : NULL;
+        a->tls_host_override =
+            acc->tls_host_override ? xstrdup(acc->tls_host_override) : NULL;
         a->logfile = acc->logfile ? xstrdup(acc->logfile) : NULL;
         a->logfile_time_format =
             acc->logfile_time_format ? xstrdup(acc->logfile_time_format) : NULL;
@@ -185,10 +190,12 @@ account_t *account_copy(account_t *acc)
         a->aliases = acc->aliases ? xstrdup(acc->aliases) : NULL;
         a->proxy_host = acc->proxy_host ? xstrdup(acc->proxy_host) : NULL;
         a->proxy_port = acc->proxy_port;
-        a->add_missing_from_header = acc->add_missing_from_header;
-        a->add_missing_date_header = acc->add_missing_date_header;
+        a->set_from_header = acc->set_from_header;
+        a->set_date_header = acc->set_date_header;
         a->remove_bcc_headers = acc->remove_bcc_headers;
+        a->undisclosed_recipients = acc->undisclosed_recipients;
         a->source_ip = acc->source_ip ? xstrdup(acc->source_ip) : NULL;
+        a->socketname = acc->socketname ? xstrdup(acc->socketname) : NULL;
     }
     return a;
 }
@@ -224,6 +231,7 @@ void account_free(void *a)
         free(p->tls_sha1_fingerprint);
         free(p->tls_md5_fingerprint);
         free(p->tls_priorities);
+        free(p->tls_host_override);
         free(p->dsn_return);
         free(p->dsn_notify);
         free(p->logfile);
@@ -232,6 +240,7 @@ void account_free(void *a)
         free(p->aliases);
         free(p->proxy_host);
         free(p->source_ip);
+        free(p->socketname);
         free(p);
     }
 }
@@ -290,21 +299,26 @@ account_t *find_account_by_envelope_from(list_t *acc_list, const char *from)
 
 
 /*
- * is_on(), is_off()
+ * is_on(), is_off(), is_auto()
  *
  * see conf.h
  */
 
-int is_on(char *s)
+int is_on(const char *s)
 {
     return (strcmp(s, "on") == 0);
 }
 
-int is_off(char *s)
+int is_off(const char *s)
 {
     return (strcmp(s, "off") == 0);
 }
 
+int is_auto(const char *s)
+{
+    return (strcmp(s, "auto") == 0);
+}
+
 
 /*
  * get_pos_int()
@@ -665,6 +679,12 @@ void override_account(account_t *acc1, account_t *acc2)
         acc1->tls_priorities = acc2->tls_priorities
             ? xstrdup(acc2->tls_priorities) : NULL;
     }
+    if (acc2->mask & ACC_TLS_HOST_OVERRIDE)
+    {
+        free(acc1->tls_host_override);
+        acc1->tls_host_override = acc2->tls_host_override
+            ? xstrdup(acc2->tls_host_override) : NULL;
+    }
     if (acc2->mask & ACC_DSN_RETURN)
     {
         free(acc1->dsn_return);
@@ -679,6 +699,10 @@ void override_account(account_t *acc1, account_t *acc2)
     {
         acc1->remove_bcc_headers = acc2->remove_bcc_headers;
     }
+    if (acc2->mask & ACC_UNDISCLOSED_RECIPIENTS)
+    {
+        acc1->undisclosed_recipients = acc2->undisclosed_recipients;
+    }
     if (acc2->mask & ACC_LOGFILE)
     {
         free(acc1->logfile);
@@ -709,19 +733,24 @@ void override_account(account_t *acc1, account_t *acc2)
     {
         acc1->proxy_port = acc2->proxy_port;
     }
-    if (acc2->mask & ACC_ADD_MISSING_FROM_HEADER)
+    if (acc2->mask & ACC_SET_FROM_HEADER)
     {
-        acc1->add_missing_from_header = acc2->add_missing_from_header;
+        acc1->set_from_header = acc2->set_from_header;
     }
-    if (acc2->mask & ACC_ADD_MISSING_DATE_HEADER)
+    if (acc2->mask & ACC_SET_DATE_HEADER)
     {
-        acc1->add_missing_date_header = acc2->add_missing_date_header;
+        acc1->set_date_header = acc2->set_date_header;
     }
     if (acc2->mask & ACC_SOURCE_IP)
     {
         free(acc1->source_ip);
         acc1->source_ip = acc2->source_ip ? xstrdup(acc2->source_ip) : NULL;
     }
+    if (acc2->mask & ACC_SOCKET)
+    {
+        free(acc1->socketname);
+        acc1->socketname = acc2->socketname ? xstrdup(acc2->socketname) : NULL;
+    }
     acc1->mask |= acc2->mask;
 }
 
@@ -734,7 +763,7 @@ void override_account(account_t *acc1, account_t *acc2)
 
 int check_account(account_t *acc, int sendmail_mode, char **errstr)
 {
-    if (!acc->host)
+    if (!acc->host && !acc->socketname)
     {
         *errstr = xasprintf(_("host not set"));
         return CONF_ESYNTAX;
@@ -749,6 +778,11 @@ int check_account(account_t *acc, int sendmail_mode, char **errstr)
         *errstr = xasprintf(_("envelope-from address is missing"));
         return CONF_ESYNTAX;
     }
+    if (acc->tls && !(acc->host || acc->tls_host_override))
+    {
+        *errstr = xasprintf(_("host not set"));
+        return CONF_ESYNTAX;
+    }
     if (acc->tls_key_file && !acc->tls_cert_file)
     {
         *errstr = xasprintf(_("tls_key_file requires tls_cert_file"));
@@ -1616,6 +1650,19 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
                 acc->tls_priorities = xstrdup(arg);
             }
         }
+        else if (strcmp(cmd, "tls_host_override") == 0)
+        {
+            acc->mask |= ACC_TLS_HOST_OVERRIDE;
+            free(acc->tls_host_override);
+            if (*arg == '\0')
+            {
+                acc->tls_host_override = NULL;
+            }
+            else
+            {
+                acc->tls_host_override = xstrdup(arg);
+            }
+        }
         else if (strcmp(cmd, "dsn_return") == 0)
         {
             acc->mask |= ACC_DSN_RETURN;
@@ -1779,16 +1826,20 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
                 }
             }
         }
-        else if (strcmp(cmd, "add_missing_from_header") == 0)
+        else if (strcmp(cmd, "set_from_header") == 0)
         {
-            acc->mask |= ACC_ADD_MISSING_FROM_HEADER;
-            if (*arg == '\0' || is_on(arg))
+            acc->mask |= ACC_SET_FROM_HEADER;
+            if (*arg == '\0' || is_auto(arg))
             {
-                acc->add_missing_from_header = 1;
+                acc->set_from_header = 2;
+            }
+            else if (is_on(arg))
+            {
+                acc->set_from_header = 1;
             }
             else if (is_off(arg))
             {
-                acc->add_missing_from_header = 0;
+                acc->set_from_header = 0;
             }
             else
             {
@@ -1799,16 +1850,16 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
                 break;
             }
         }
-        else if (strcmp(cmd, "add_missing_date_header") == 0)
+        else if (strcmp(cmd, "set_date_header") == 0)
         {
-            acc->mask |= ACC_ADD_MISSING_DATE_HEADER;
-            if (*arg == '\0' || is_on(arg))
+            acc->mask |= ACC_SET_DATE_HEADER;
+            if (*arg == '\0' || is_auto(arg))
             {
-                acc->add_missing_date_header = 1;
+                acc->set_date_header = 2;
             }
             else if (is_off(arg))
             {
-                acc->add_missing_date_header = 0;
+                acc->set_date_header = 0;
             }
             else
             {
@@ -1839,6 +1890,26 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
                 break;
             }
         }
+        else if (strcmp(cmd, "undisclosed_recipients") == 0)
+        {
+            acc->mask |= ACC_UNDISCLOSED_RECIPIENTS;
+            if (*arg == '\0' || is_on(arg))
+            {
+                acc->undisclosed_recipients = 1;
+            }
+            else if (is_off(arg))
+            {
+                acc->undisclosed_recipients = 0;
+            }
+            else
+            {
+                *errstr = xasprintf(
+                        _("line %d: invalid argument %s for command %s"),
+                        line, arg, cmd);
+                e = CONF_ESYNTAX;
+                break;
+            }
+        }
         else if (strcmp(cmd, "source_ip") == 0)
         {
             acc->mask |= ACC_SOURCE_IP;
@@ -1852,6 +1923,61 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
                 acc->source_ip = xstrdup(arg);
             }
         }
+        else if (strcmp(cmd, "socket") == 0)
+        {
+            acc->mask |= ACC_SOCKET;
+            free(acc->socketname);
+            if (*arg == '\0')
+            {
+                acc->socketname = NULL;
+            }
+            else
+            {
+                acc->socketname = xstrdup(arg);
+            }
+        }
+        else if (strcmp(cmd, "add_missing_from_header") == 0)
+        {
+            /* compatibility with < 1.8.8 */
+            acc->mask |= ACC_SET_FROM_HEADER;
+            if (*arg == '\0' || is_on(arg))
+            {
+                acc->set_from_header = 2;
+            }
+            else if (is_off(arg))
+            {
+                acc->set_from_header = 0;
+            }
+            else
+            {
+                *errstr = xasprintf(
+                        _("line %d: invalid argument %s for command %s"),
+                        line, arg, cmd);
+                e = CONF_ESYNTAX;
+                break;
+            }
+        }
+        else if (strcmp(cmd, "add_missing_date_header") == 0)
+        {
+            /* compatibility with < 1.8.8 */
+            acc->mask |= ACC_SET_DATE_HEADER;
+            if (*arg == '\0' || is_on(arg))
+            {
+                acc->set_date_header = 2;
+            }
+            else if (is_off(arg))
+            {
+                acc->set_date_header = 0;
+            }
+            else
+            {
+                *errstr = xasprintf(
+                        _("line %d: invalid argument %s for command %s"),
+                        line, arg, cmd);
+                e = CONF_ESYNTAX;
+                break;
+            }
+        }
         else if (strcmp(cmd, "auto_from") == 0)
         {
             /* compatibility with <= 1.8.7 */
