@@ -79,9 +79,10 @@
 #define ACC_SET_FROM_HEADER             (1LL << 31LL)
 #define ACC_SET_DATE_HEADER             (1LL << 32LL)
 #define ACC_REMOVE_BCC_HEADERS          (1LL << 33LL)
-#define ACC_SOURCE_IP                   (1LL << 34LL)
-#define ACC_LOGFILE_TIME_FORMAT         (1LL << 35LL)
-#define ACC_SOCKET                      (1LL << 36LL)
+#define ACC_UNDISCLOSED_RECIPIENTS      (1LL << 34LL)
+#define ACC_SOURCE_IP                   (1LL << 35LL)
+#define ACC_LOGFILE_TIME_FORMAT         (1LL << 36LL)
+#define ACC_SOCKET                      (1LL << 37LL)
 
 typedef struct
 {
@@ -137,6 +138,7 @@ typedef struct
     int set_from_header;        /* 0=off, 1=on, 2=auto */
     int set_date_header;        /* 0=off,       2=auto */
     int remove_bcc_headers;     /* remove the Bcc headers from the mail? */
+    int undisclosed_recipients; /* remove To, Cc, Bcc, add anonymous To */
     /* source ip binding */
     char *source_ip;            /* Source IP to bind the connection to */
     /* unix domain socket */
