@@ -3,7 +3,7 @@
  *
  * This file is part of msmtp, an SMTP client, and of mpop, a POP3 client.
  *
- * Copyright (C) 2004, 2005, 2011, 2014, 2018
+ * Copyright (C) 2004, 2005, 2011, 2014, 2018, 2020
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -126,6 +126,23 @@ char *xstrndup(const char *s, size_t n)
  * xasprintf()
  */
 
+#ifndef HAVE_VASPRINTF
+static int vasprintf(char **strp, const char *format, va_list args)
+{
+    /* This replacement function only works when the vsnprintf() function is available
+     * and its return value is standards compliant. This is true for the MinGW version
+     * of vsnprintf(), but not for Microsofts version (Visual Studio etc.)!
+     */
+    int length = vsnprintf(NULL, 0, format, args);
+    if (length > INT_MAX - 1 || !(*strp = malloc(length + 1)))
+    {
+        return -1;
+    }
+    vsnprintf(*strp, length + 1, format, args);
+    return length;
+}
+#endif
+
 char *xasprintf(const char *format, ...)
 {
     char *strp;
